# CALCULATION OF STATISTICS
# averaging all stations

# inputs:
#   local_stations
#   df_obs_sub
#   model_raw_at_AQMS_df
#   LOOCV_total_df

# outputs:
#   Summary_stats (png+RDS)
#   Summary_stats_h (png+RDS)
#   COE and MB mapshots

library(tidyr)
library(dplyr)
library(openair)
library(ggplot2)

args = commandArgs(TRUE)
if (length(args) %in% c(2,3)) {
  pollutant <- args[1]
  year <- args[2]
  if(length(args)==3 & tolower(args[3])=="gbm"){
    with_gbm <- TRUE
  }else{
    with_gbm <- FALSE
  }
} else {
  #cat("\n***Arguments missing***\n Arguments: pollutant year \n Pollutant codes: no2, o3, pm10, pm2p5, so2, co")
  stop("\n***Arguments missing***\n Arguments: pollutant year \n Pollutant codes: no2, o3, pm10, pm2p5, so2, co\n Optionally 3rd argument: gbm")
}
print(args)

# PARAMETERS ----
input_path <- "/esarchive/scratch/ccarnere/RScripts/"
output_path <- "/esarchive/scratch/ccarnere/AQS_database/postprocessed/regional/statistics/"
# pollutant <- "NO2"
# year <- 2019

# MAIN----


#TODO implement statistics for GBM (PM2.5)


pollutant <- tolower(pollutant)
format <- ifelse(pollutant %in% c("so2","co"), "ghost", "gene")

output_path <- ifelse(!pollutant %in% c("pm10","pm2p5"), 
                      paste0(output_path,"sconc",pollutant,"/"),
                      paste0(output_path,pollutant,"/"))

# read input data ----
local_stations <- readRDS(paste0(input_path,"results_regional_",year,"/results_gene_",pollutant,
                                 "/output/local_stations_",format,"_",pollutant,".rds"))
df_obs_sub <- readRDS(paste0(input_path,"results_regional_",year,"/results_gene_",pollutant,
                             "/output/df_obs_",format,"_",pollutant,".rds"))
model_raw_at_AQMS_df <- readRDS(paste0(input_path,"results_regional_",year,"/results_gene_",pollutant,
                                       "/output/model_raw_at_AQMS_",format,"_",pollutant,"_df.rds"))
LOOCV_total_df <- readRDS(paste0(input_path,"results_regional_",year,"/results_gene_",pollutant,
                                 "/output/LOOCV_",format,"_",pollutant,
                                 ifelse(with_gbm,"_gbm",""),"_df.rds"))

# TODO for ghost, rename the stations in LOOCV to match those in local_stations


# generate a list with the names in local stations that are in df_obs_sub:
names_local_stations <- as.list(intersect(local_stations@data$ref_obs, colnames(df_obs_sub)))

#We need a data.frame with raw_model and LOOCV_model to apply the stadistics

# Hourly stats per station ----
raw_model_total         <- NULL
UK_model_total          <- NULL 
geostats_results        <- NULL

for (station_code in names_local_stations) {
  # station_code <- names_local_stations[[1]]

  # check that we have the model interpolated at that station 
  if(station_code %in% names(model_raw_at_AQMS_df)){
  
    # merge observations with model for each station
      
    # raw model
    raw_model_estacio <- full_join(df_obs_sub[,c("date",station_code)], 
                     model_raw_at_AQMS_df[,c("date",station_code)], 
                     by="date")
    names(raw_model_estacio) <- c("date","obs","mod")
    
    raw_model_estacio <- raw_model_estacio %>% 
      mutate(model="raw_model",
             station=station_code)
    
    # UK
    UK_model_estacio <- full_join(df_obs_sub[,c("date",station_code)], 
                     LOOCV_total_df[,c("date",station_code)], 
                     by="date")
    names(UK_model_estacio) <- c("date","obs","mod")
    
    UK_model_estacio <- UK_model_estacio %>% 
      mutate(model="UK_model",
             station=station_code)
    
  
    # Statistics for a single station
    Raw_geostats    <- openair::modStats(raw_model_estacio, obs = "obs", mod = "mod", type = "model")
    UK_geostats     <- openair::modStats(UK_model_estacio, obs = "obs", mod = "mod", type = "model")
    
    geostats_station <- data.frame(x=local_stations@coords[which(local_stations@data$ref_obs == station_code),1],
                                   y=local_stations@coords[which(local_stations@data$ref_obs == station_code),2],
                                   COE_raw = Raw_geostats$COE,
                                   COE_UK = UK_geostats$COE,
                                   MB_raw = Raw_geostats$MB,
                                   MB_UK = UK_geostats$MB)
    
    # bind all stations
    geostats_results    <- rbind(geostats_results,geostats_station)
    #raw_model_total and corr_model_total are a data frame with three lists (obs, mod, model(raw or corrected))
    raw_model_total     <- rbind(raw_model_total,raw_model_estacio) 
    UK_model_total      <- rbind(UK_model_total,UK_model_estacio)
    
  }
}

mod_obs_data <- full_join(raw_model_total,
                          UK_model_total)

saveRDS(mod_obs_data,
        paste0(output_path,"mod_obs_stations_",pollutant,"_",year,".RDS"))

Summary_stats <- openair::modStats(mod_obs_data, obs = "obs", mod = "mod", type = "model")

Summary_stats <- Summary_stats %>%
  bind_cols(data.frame("pol" = pollutant, 
                          "year"=year,
                          "aggreg"="hourly"), 
            .)


# Hourly average all stations -----

mod_obs_data_mean <- mod_obs_data %>%
  dplyr::group_by(date,model) %>%
  dplyr::summarise(obs = mean(obs, na.rm=T),
            mod = mean(mod, na.rm=T))
# transform NaNs to NAs
mod_obs_data_mean <- mod_obs_data_mean %>% 
  mutate_all(~ifelse(is.nan(.), NA, .))




Summary_stats_mean <- openair::modStats(mod_obs_data_mean, obs = "obs", mod = "mod", type = "model")

# add column pollutant and year
Summary_stats_mean <- Summary_stats_mean %>%
  bind_cols(data.frame("pol" = pollutant, 
                        "year"=year,
                        "aggreg"="all_stations_hourly"), 
            .)

summary_stats_all <- rbind(Summary_stats, Summary_stats_mean)

saveRDS(summary_stats_all,
        paste0(output_path,"Stats_",pollutant,"_",year,".RDS"))


# Plot model vs obs ----
# params
text_size=15
cols <- c("#003F7D","orange","brown")

# merge and transform data
prefix_cols <- ifelse(format=="gene","ES","STA")

df_obs <- df_obs_sub %>% pivot_longer(cols=starts_with(prefix_cols), names_to = "station")
df_raw <- model_raw_at_AQMS_df %>% pivot_longer(cols=starts_with(prefix_cols), names_to = "station")
df_UK <- LOOCV_total_df %>% pivot_longer(cols=starts_with(prefix_cols), names_to = "station")

df <- full_join(df_obs, df_raw, by=c("date","station"))
df <- full_join(df, df_UK, by=c("date","station"))
names(df) <- c("date","station","obs","raw","UK")


# annual means
df_annual <- df %>%
  dplyr::group_by(station) %>%
  dplyr::summarise(across(c(obs,raw,UK), ~mean(.,na.rm=T)))
df_annual[df_annual=="NaN"] <- NA

#... legislated limit values ----
limits_directive_hourly <- data.frame("no2"=200,
                               "pm10"=50,
                               "so2"=350)
vars_hourly <- names(limits_directive_hourly)
cond_hourly <- limits_directive_hourly %>% slice(1)

limits_directive_annual <- data.frame("no2"=40,
                               "pm10"=40,
                               "pm2p5"=25)
vars_annual <- names(limits_directive_annual)
cond_annual <- limits_directive_annual %>% slice(1)


# set limits to the data
limits <- data.frame("no2" = 750,
                   "o3" = 400, 
                   "pm10" = 1000, 
                   "pm2p5" = 200, 
                   "so2" = 750,
                   "co" = 5000)

limit <- limits[pollutant] %>% pull()

df[,-c(1,2)][df[,-c(1,2)] >limit] <- NA
df[,-c(1,2)][df[,-c(1,2)] <0] <- NA


# plot hourly/daily
for(i in 1:length(vars_hourly)){
  
  if(grepl(tolower(pollutant), vars_hourly[i], fixed=T)){

    p <- ggplot() +
      geom_point(data=df, aes(x=obs, y=raw, colour="c1")) +
      geom_point(data=df, aes(x=obs, y=UK, colour="c2")) +
      geom_hline(yintercept=cond_hourly[,i], linetype="dashed")+
      geom_vline(xintercept=cond_hourly[,i], linetype="dashed")+
      labs(y="model",
           x="observation") +
      scale_color_manual(name = "model",
                         breaks = c("c1", "c2"),
                         values = cols,
                         labels = c("raw", "UK"))+
      # ylim(0,max(max(df$raw,na.rm=T),max(df$UK, na.rm=T)))+
      # xlim(0,max(df$obs, na.rm=T))+
      coord_fixed(ratio=1)+
      theme_classic()+
      ggtitle(paste0(year, " - ", toupper(pollutant), 
                     ifelse(pollutant %in% c("pm10","pm2p5"), " daily", " hourly")," mean"))+
      theme(aspect.ratio=1,
            text = element_text(size=text_size),
            panel.grid.major = element_blank(), 
            panel.background = element_rect(colour = "black", size=0.5))
    
    # save plot----
    ggsave(file=paste0(output_path,"model_vs_obs_",pollutant,"_",year,".png"),
           plot = p,
           width=8,
           height=8,
           units="in",
           dpi = 300)
  }
}

# plot annual
for(i in 1:length(vars_annual)){
  
  if(grepl(tolower(pollutant), vars_annual[i], fixed=T)){
    print(vars_annual[i])
    
    p <- ggplot() +
      geom_point(data=df_annual, aes(x=obs, y=raw, colour="c1")) +
      geom_point(data=df_annual, aes(x=obs, y=UK, colour="c2")) +
      geom_hline(yintercept=cond_annual[,i], linetype="dashed")+
      geom_vline(xintercept=cond_annual[,i], linetype="dashed")+
      labs(y="model",
           x="observation") +
      scale_color_manual(name = "model",
                         breaks = c("c1", "c2"),
                         values = cols,
                         labels = c("raw", "UK"))+
      # ylim(0,max(max(df_annual$raw,na.rm=T),max(df_annual$UK, na.rm=T)))+
      # xlim(0,max(df_annual$obs, na.rm=T))+
      coord_fixed()+
      theme_classic()+
      ggtitle(paste0(year, " - ", toupper(pollutant), " annual mean"))+
      theme(aspect.ratio=1,
            text = element_text(size=text_size),
            panel.grid.major = element_blank(), 
            panel.background = element_rect(colour = "black", size=0.5))
    
    # save plot----
    ggsave(file=paste0(output_path,"model_vs_obs_",pollutant,"_",year,"_annual.png"),
           plot = p,
           width=8,
           height=8,
           units="in",
           dpi = 300)
  }
}
